/**
* Copyright (c) 2003-2006 Illinois Institute of Technology.
* All rights reserved.
*
* This file is part of the NBP software package. For license
* information, see the LICENSE file in the top level directory of the
* NBP source distribution.
*
* Released Date: September 5 2006
* This is the NBP I release by the
* NBP Group,
* http://www.cs.iit.edu/~nbp
* Department of Computer Science,
* Illinois Institute of Technology.
*
* This release has been tested under Windows XP Professional
*
* Supervisor
* ----------
* Illiniois Institute of Technology:
* -Dr. Xian-He Sun
*
* Author(s)
* -----------------
* Illinois Institute of Technology:
* -Alaknantha Eswaradass
* -Ming Wu
* -Xian-He Sun
*
*/


import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
/**
 *
 * Gets input data from users either via file or database and invokes 
 * Pre processor and Neural Network predictor
 */
public class ActionPanel extends JPanel{
	JLabel lblHeading = new JLabel("Get Data");
	FilePanel fPanel = new FilePanel();
	DBPanel  dbPanel = new DBPanel();
	JButton btnPredict = new JButton("Predict");
	JButton btnClose   = new JButton("Cancel");
	private GUI gui = null;
	ActionListener cancelListener = new ActionListener(){
		public void actionPerformed(ActionEvent e) {
			gui.backToMain();
		}
	};
	
	public ActionPanel(GUI gui){
		setPreferredSize(new Dimension(300,450));
		this.gui = gui;
		JPanel  btnPanel = new JPanel();
		btnPanel.setLayout(new FlowLayout());
		btnPanel.add(btnPredict);
		btnPanel.add(btnClose);
		setLayout(new BoxLayout(this,BoxLayout.Y_AXIS));
		add(Box.createVerticalStrut(20));
		JPanel pnl = new JPanel();
		pnl.setLayout(new BoxLayout(pnl,BoxLayout.X_AXIS));
		pnl.add(lblHeading);
		pnl.add(Box.createHorizontalStrut(250));
		add(pnl);
		add(Box.createVerticalStrut(20));
		add(fPanel);
		add(Box.createVerticalStrut(20));
		add(dbPanel);
		add(btnPanel);
		btnPredict.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				System.out.println("predict pressed");
				invokePredictor();
			}
		});
		btnClose.addActionListener(cancelListener);
	}
	
	
	/**
	 *  
	 *  Invokes neural network predictor
	 * 
	 *  @param none
	 *  @return none
	 *
	 */
	private void invokePredictor(){
		String fName = fPanel.getFileName();
		PredictorRunner runner =  null;
		if(fName==null || fName.length()==0){
			fName = "temp"+dbPanel.getStatId()+".txt";
			runner = new PredictorRunner(gui,dbPanel.getStatId(),dbPanel.getConfig());
		}else{
			runner=new PredictorRunner(gui,dbPanel.getConfig(),fName);
		}

		System.out.println(fPanel.getFileName()+"::invoke later");
		javax.swing.SwingUtilities.invokeLater(runner);
		System.out.println("after returning from invokelatter");
		gui.displayPanel(GUI.PANEL_RESULTS);
		
	}
	
	public static void main(String argv[]){
		   JFrame frame = new JFrame("Forecast");
		   frame.setVisible(true);
		   frame.setSize(500, 500);
		  
		   frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		   ActionPanel nnPanel = new ActionPanel(null);
		   frame.getContentPane().setLayout(new BorderLayout());
		   frame.getContentPane().add(nnPanel,BorderLayout.CENTER);
		   frame.validate();
	  }
}
