/**
* Copyright (c) 2003-2006 Illinois Institute of Technology.
* All rights reserved.
*
* This file is part of the NBP software package. For license
* information, see the LICENSE file in the top level directory of the
* NBP source distribution.
*
* Released Date: September 5 2006
* This is the NBP I release by the
* NBP Group,
* http://www.cs.iit.edu/~nbp
* Department of Computer Science,
* Illinois Institute of Technology.
*
* This release has been tested under Windows XP Professional
*
* Supervisor
* ----------
* Illiniois Institute of Technology:
* -Dr. Xian-He Sun
*
* Author(s)
* -----------------
* Illinois Institute of Technology:
* -Alaknantha Eswaradass
* -Ming Wu
* -Xian-He Sun
*
*/


import java.util.*;
import java.io.*;
import weka.classifiers.functions.neural.*;
import weka.core.*;

/*******************************************************************************************************
 *    @date Modifed   : 3/4/2005
 *    @version        : 2.0
 *
 *
 * This code implements a bandwidth prediction system.
 *
 * Input: Various parameters affecting the Available Bandwidth metric like time unit,
 * Minimum number of bits in any one second, Maximum number of bits in any one second,
 * Average number of bits in past 1 bin, Average number of bits in past 2 bin,
 * Average number of bits in past 3 bin, Average number of bits in past 4 bin,
 * Average number of bits in past 5 bin, Average number of bits in past 5 bin,
 * Average number of bits, Average number of bits to predict
 * Output: The Prediction results
 *
 *
 * This Code uses the neural network given by weka. First trains the first half of the input data and
 * predicts the second half.
*******************************************************************************************************/


public class BandwidthPredictor //extends Machine implements common
{
	/** neural network components */
	NeuralNetwork nn;
	Instances m_instances;
	Instance m_instance;
	int num_att;
	int numOfLine;
        String fileName = ""; //will be set by command line arguments..
        String outFileName = ""; //will be set by command line argumetns..



	public BandwidthPredictor(String set[],String inputFileName,String outFileName) {
          try{
            this.fileName=inputFileName;
            this.outFileName=outFileName;
            get_linenum();
            nn = new NeuralNetwork();
            make_templates(set);
            initialize_nn();
            make_instances_from_file();
            nn.buildClassifier(m_instances);
            System.out.println(nn);
	 }catch(Exception e){
           e.printStackTrace();
         }
	}

	public BandwidthPredictor(String set[],String inputFileName,String outFileName,Config config) {
          try{
            this.fileName=inputFileName;
            this.outFileName=outFileName;
            get_linenum();
            nn = new NeuralNetwork();
            make_templates(set);
            initialize_nn(config);
            make_instances_from_file();
            nn.buildClassifier(m_instances);
            System.out.println(nn);
	 }catch(Exception e){
           e.printStackTrace();
         }
	}


	public void get_estimate(){
		int i,j;
		double db;
		String line;
		double estimated_time, prederror=0, maxerror=0, minerror=1, tmperror, averprederror = 0;
                double errorstd=0, tmperrorstd=0;
                PrintWriter out = null;

		try{
			BufferedReader in = new BufferedReader(new FileReader(fileName));
                        out = new PrintWriter(new FileWriter(outFileName));
			line = in.readLine();
			j = 1;
			prederror = 0;
                        int countLines =0;
			while ( line != null ){
			  if ( j >= numOfLine/2){
			  	Instance instance = new Instance(m_instance);
				StringTokenizer st = new StringTokenizer(line);
				i = 0;
                                int index=0;
                                long timeInst = 0;
				while(st.hasMoreTokens()){
					db = Double.parseDouble(st.nextToken());
                                        if(index==0) timeInst =(long) db;
                                        if(use[index]){
                                          instance.setValue(i, db);
                                          i++;
                                        }
                                        index++;
				}
				instance.setDataset(m_instances);
				java.text.NumberFormat format = java.text.NumberFormat.getNumberInstance();
                                try{
                                  estimated_time = nn.classifyInstance(instance);
                                  tmperror = Math.abs(estimated_time - instance.value(i - 1)) /
                                      instance.value(i - 1);
                                   countLines++;
                                   out.println((long)instance.value(0)+ " " + instance.value(i - 1)  + " " +  estimated_time + " " +
                                               tmperror);
                                   prederror += tmperror;
                                   if (maxerror < tmperror)
                                     maxerror = tmperror;
                                   if (minerror > tmperror)
                                     minerror = tmperror;

                                }catch(Exception e){
                                  e.printStackTrace();
                                }
			  }
			  line = in.readLine();
			  j++;
			}
			in.close();
                        averprederror = prederror/countLines;
                        System.out.println("average prediction error is "+ averprederror);
			//out.println("average prediction error is "+ averprederror);
			//out.println("max prediction error is "+ maxerror);
			//out.println("min prediction error is "+ minerror);
                        out.close();
                      }catch ( IOException iox ){
			iox.printStackTrace();
                      }catch(Exception e){
                        e.printStackTrace();
                      }finally{
                        if(out!=null) out.close();
                      }

		try{
			BufferedReader in = new BufferedReader(new FileReader(fileName));
			line = in.readLine();
			j = 1;
			while ( line != null ){
			  if ( j >= numOfLine/2){
				Instance instance = new Instance(m_instance);
				StringTokenizer st = new StringTokenizer(line);
				i = 0;
                                int index=0;
				while(st.hasMoreTokens()){
					db = Double.parseDouble(st.nextToken());
                                        if(use[index]){
                                          instance.setValue(i, db);
                                          i++;
                                        }
                                        index++;
				}
				instance.setDataset(m_instances);
				try{
					estimated_time = nn.classifyInstance(instance);
					tmperror = Math.abs(estimated_time - instance.value(i-1))/instance.value(i-1);
					tmperrorstd = (tmperror-averprederror)*(tmperror-averprederror);
					errorstd += tmperrorstd;
				}catch(Exception e){
					e.printStackTrace();
				}
			  }
			  line = in.readLine();
			  j++;

			}
			in.close();
                        System.out.println("prediction error std is "+ Math.sqrt(tmperrorstd/(numOfLine-numOfLine/2+1)));
		}catch ( IOException iox ){
			iox.printStackTrace();
		}
	}



	public void make_instances_from_file() {
		double db;
		int i,j;
		String line;
		try{
			BufferedReader in = new BufferedReader(new FileReader(fileName));
			line = in.readLine();
			j=0;
			while ( line != null ) {
			  if (j < numOfLine/2) {
				Instance instance = new Instance(m_instance);
				StringTokenizer st = new StringTokenizer(line);
				i = 0;
                                int index=0;
				while(st.hasMoreTokens()) {
                                        String str=st.nextToken();
                                        if(str==null){
                                          System.out.println("Null line.."+line);
                                          continue;
                                        }
					db = Double.parseDouble(str);
                                        if(use[index]){
                                          instance.setValue(i, db);
                                          i++;
                                        }
                                        index++;
				}
				instance.setDataset(m_instances);
				m_instances.add(instance);
			  }
			  line = in.readLine();
			  j++;
			}
			in.close();
		}catch ( IOException iox ){
			iox.printStackTrace();
		}
	}




	public void get_linenum() {
		int j=0;
		String line;
		try{
			BufferedReader in = new BufferedReader(new FileReader(fileName));
			line = in.readLine();
			while ( line != null ){
                          line = in.readLine(); j++;}
			numOfLine = j;
			in.close();
		}catch ( IOException iox ){
			iox.printStackTrace();
		}
	}



	// templates for creaton of neural network data
	public void make_templates(String set[]){
		FastVector fv;
		num_att = 0;
                for(int i=0;i<use.length;i++)
                  if(use[i]) num_att++;
                int index =0;
		Attribute[] att = new Attribute[num_att];
                for(int i=0;i<set.length;i++)
                  if(use[i])
                    att[index++] = new Attribute(set[i]);
		fv = new FastVector();
		for(int i=0;i<num_att;i++)
		  fv.addElement(att[i]);
		m_instances = new Instances("instances1",fv,2000);
		m_instances.setClassIndex(m_instances.numAttributes() - 1);
		m_instance = new Instance(num_att);
		m_instance.setDataset(m_instances);
	}



	//initialize neural network
	public void initialize_nn(){
		String[] options = new String[6];
		options[0] = "-H";	options[1] = "3";
		options[2] = "-N";	options[3] = "700";
		options[4] = "-E";	options[5] = "2";
		try{
			nn.setOptions(options);
		}catch(Exception e){
			System.out.println("exception: "+e);
		}
	}
	
//	initialize neural network
	public void initialize_nn(Config config){
		String[] options = new String[6];
		options[0] = "-H";	options[1] =config.nnConfig.noOFHiddenLayers+"";
		options[2] = "-N";	options[3] =config.nnConfig.maxOfEpochs+"";
		options[4] = "-L";	options[5] =config.nnConfig.learningRate+"";
		try{
			nn.setOptions(options);
		}catch(Exception e){
			System.out.println("exception: "+e);
		}
	}





    public static void main(String[] args){
    if(args.length<2) {
            System.out.println("BandwidthPredictor's input parameters inputfile, output file");
            System.exit(-1);
    }
    BandwidthPredictor bp = new BandwidthPredictor(
    		Columns ,
          args[0],args[1] );
   bp.get_estimate();
}

    static String[] Columns = {"timestamp", "month", "day", "hour", "minute",
            "min_no_of_bits", "max_no_of_bits",
	          "avg_no_of_bits_past1bin", "avg_no_of_bits_past2bin","avg_no_of_bits_past3bin",
	          "avg_no_of_bits_past4bin", "avg_no_of_bits_past5bin", "avg_no_of_bits_past6bin",
	          "avg_no_of_bits", "avg_no_of_bits_to_predict"} ; 


    static boolean[] use={
           true,
           true,
           true,
           true,
           true,
           true,
           true,
           true,
           true,
           true,
           true,
           true,
           true,
           true,
           true
            };

    // static String nwConf, tType, pType;
     static long binSize = 0;
}
