/**

* Copyright (c) 2003-2006 Illinois Institute of Technology.
* All rights reserved.
*
* This file is part of the NBP software package. For license
* information, see the LICENSE file in the top level directory of the
* NBP source distribution.
*
* Released Date: September 5 2006
* This is the NBP I release by the
* NBP Group,
* http://www.cs.iit.edu/~nbp
* Department of Computer Science,
* Illinois Institute of Technology.
*
* This release has been tested under Windows XP Professional
*
* Supervisor
* ----------
* Illiniois Institute of Technology:
* -Dr. Xian-He Sun
*
* Author(s)
* -----------------
* Illinois Institute of Technology:
* -Alaknantha Eswaradass
* -Ming Wu
* -Xian-He Sun
*
*/
import java.awt.Paint;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.image.BufferedImage;

import org.jCharts.Chart;
import org.jCharts.axisChart.AxisChart;
import org.jCharts.chartData.AxisChartDataSet;
import org.jCharts.chartData.ChartDataException;
import org.jCharts.chartData.DataSeries;
import org.jCharts.properties.AxisProperties;
import org.jCharts.properties.DataAxisProperties;
import org.jCharts.properties.ChartProperties;
import org.jCharts.properties.LegendProperties;
import org.jCharts.properties.LineChartProperties;
import org.jCharts.properties.PointChartProperties;
import org.jCharts.properties.PropertyException;
import org.jCharts.test.TestDataGenerator;
import org.jCharts.types.ChartType;


/**
 *
 * Generates a line chart between measured and predicted bandwidth
 *  over different timestamp
 */
public class ChartGenerator {
	
	/**
	 * Generates chart object from resultset datamodel 
	 * @param dataModel
	 * @return Generated chart object
	 * @throws ChartDataException
	 */
	
	public Chart generateChart(ResultDataModel dataModel) throws ChartDataException
	{
		int dataSize = dataModel.getRowCount();
		String[] xAxisLabels= new String[dataSize];
		for(int i=0;i<dataSize;i++){
			xAxisLabels[i]=(String) dataModel.getValueAt(i,ResultDataModel.COL_TIME);
		}
		String xAxisTitle= "Time(milliseconds)";
		String yAxisTitle= "Bandwidth";
		String title= "Bandwidth Prediction Results";
		DataSeries dataSeries = new DataSeries( xAxisLabels, xAxisTitle, yAxisTitle, title );

		double[][] data= new double[2][dataSize];
		for(int i=0;i<dataSize;i++){
			data[0][i]=Double.parseDouble((String) dataModel.getValueAt(i,ResultDataModel.COL_MESSURED));
			data[1][i]=Double.parseDouble((String) dataModel.getValueAt(i,ResultDataModel.COL_PREDICTED));
			
		}
		
		String[] legendLabels= { "Measured" ,"Predicted"};
		Paint[] paints= TestDataGenerator.getRandomPaints( 2 );

		Stroke[] strokes= { LineChartProperties.DEFAULT_LINE_STROKE, LineChartProperties.DEFAULT_LINE_STROKE };
		Shape[] shapes= { PointChartProperties.SHAPE_CIRCLE, PointChartProperties.SHAPE_CIRCLE};
		LineChartProperties lineChartProperties= new LineChartProperties( strokes, shapes );

		AxisChartDataSet axisChartDataSet= new AxisChartDataSet( data, legendLabels, paints, ChartType.LINE, lineChartProperties );
		dataSeries.addIAxisPlotDataSet( axisChartDataSet );

		
		ChartProperties chartProperties= new ChartProperties();
		AxisProperties axisProperties= new AxisProperties();
		LegendProperties legendProperties= new LegendProperties();
		try{
		DataAxisProperties dataAxisProperties= (DataAxisProperties) axisProperties.getYAxisProperties();
		dataAxisProperties.setUserDefinedScale( 150, 50 );
		dataAxisProperties.setNumItems( 8 );
		
		}catch(Exception e){
			e.printStackTrace();
		}
		
		AxisChart axisChart= new AxisChart( dataSeries, chartProperties, axisProperties, legendProperties, 600, 400 );

		
		
		return axisChart;
	}
	
	/**
	 *  Renders the chart model and creates image in displayable format
	 * @param chart
	 * @return BufferedImage in panel displayable format
	 * @throws ChartDataException
	 * @throws PropertyException
	 */
	
	public  BufferedImage render( Chart chart ) throws ChartDataException, PropertyException//, IOException
	{
		BufferedImage bufferedImage = null;

		//---if we use an ImageMap, we already have rendered the chart byt the time we get here so,
		//---	simply return the rendered image.
		if( chart.getGenerateImageMapFlag() )
		{
			bufferedImage = chart.getBufferedImage();
		}
		//---else, create a new BufferedImage and set the Graphics2D onto the chart.
		else
		{
			bufferedImage = new BufferedImage( chart.getImageWidth(), chart.getImageHeight(), BufferedImage.TYPE_INT_RGB );
			chart.setGraphics2D( bufferedImage.createGraphics() );
			chart.render();
		}

		return bufferedImage;
	}

}
