/*

* Copyright (c) 2003-2006 Illinois Institute of Technology.
* All rights reserved.
*
* This file is part of the NBP software package. For license
* information, see the LICENSE file in the top level directory of the
* NBP source distribution.
*
* Released Date: September 5 2006
* This is the NBP I release by the
* NBP Group,
* http://www.cs.iit.edu/~nbp
* Department of Computer Science,
* Illinois Institute of Technology.
*
* This release has been tested under Windows XP Professional
*
* Supervisor
* ----------
* Illiniois Institute of Technology:
* -Dr. Xian-He Sun
*
* Author(s)
* -----------------
* Illinois Institute of Technology:
* -Alaknantha Eswaradass
* -Ming Wu
* -Xian-He Sun
*
*/
import java.awt.*;
import javax.swing.border.*;
import javax.swing.*;
import java.awt.event.*;

/**
 * 
 * 
 * Contains main function which inturn display the main panel and 
 * displays other panels according to users action
 *
 */
public class GUI extends JFrame implements StatusBar{

   public static Config DefaultConfig = new Config();
   static{
   	DefaultConfig.nnConfig = new NNConfig();
   	DefaultConfig.fiConfig = new FIConfig();
   	DefaultConfig.nwsMonitorConfig = new NWSMonitorConfig();
   	DefaultConfig.dbConfig = new DBConfig();
   	
   	DefaultConfig.nnConfig.learningRate = 0.01f;
   	DefaultConfig.nnConfig.maxOfEpochs = 700;
   	DefaultConfig.nnConfig.noOFHiddenLayers = 3;
   	
   	DefaultConfig.fiConfig.predictionStepFrom = 1;
   	DefaultConfig.fiConfig.predictionStepTo = 1;
   	DefaultConfig.fiConfig.predictionStepStep = 1;
   	DefaultConfig.fiConfig.binSizeFrom = 100;
   	DefaultConfig.fiConfig.binSizeTo = 100;
   	DefaultConfig.fiConfig.binSizeStep = 100;
   	
   	DefaultConfig.nwsMonitorConfig.sourceMachine = "mgt5.dotresearch.org";
   	DefaultConfig.nwsMonitorConfig.destMachine = "iit01.dotresearch.org";
   	DefaultConfig.nwsMonitorConfig.noOfMeasurements = "2000";
   	DefaultConfig.nwsMonitorConfig.skillToMonitor = "tcpMessageMonitor";
   	DefaultConfig.nwsMonitorConfig.period = "1";
   	DefaultConfig.nwsMonitorConfig.sendMsgSize = "16";
   	DefaultConfig.nwsMonitorConfig.socketBufferSize = "32";  	
   	DefaultConfig.nwsMonitorConfig.totalDataSize = "64";
   	
   	DefaultConfig.dbConfig.userName="RTCR";
   	DefaultConfig.dbConfig.password="RTCR";
   	DefaultConfig.dbConfig.driverName="oracle.jdbc.driver.OracleDriver";
   	DefaultConfig.dbConfig.connectionString="jdbc:oracle:thin:@localhost:1521:rtcr";
   	
   }
   public static final int PANEL_MAIN = 0;
   public static final int PANEL_CONFIGURE = 1;
   public static final int PANEL_ACTION = 2;
   public static final int PANEL_RESULTS = 3;
   public static final int PANEL_MONITOR = 4;
   public static final int PANEL_SSH = 5;
   
   public JPanel currentPanel = null;
   public JPanel conPanel = new JPanel();
   private JFrame f = new JFrame("Neural Network Based System");
  
   JPanel statusPanel = new JPanel();
   JLabel lblStatus = new JLabel("Loading");
   private Config config = DefaultConfig;
  
   public static void main(String argv[]){
	    try {
	        UIManager.setLookAndFeel(
	            UIManager.getSystemLookAndFeelClassName());
	    } catch (Exception e) { }

       GUI app = new GUI("GUI");
       app.setVisible(true);
   } //main
   
   public JPanel getCurrentPanel(){
   	return currentPanel;
   }
   public void setConfig(Config config){
   		this.config = config;
   }
   public Config getConfig(){
   		return config;
   }
	public GUI(String title){

		super(title);
		setSize(350, 500);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        currentPanel = new MainPanel(this);
		conPanel.add(currentPanel,BorderLayout.CENTER); 
		conPanel.setSize(350,480);
        this.getContentPane().setLayout(new BorderLayout());
        this.getContentPane().add(conPanel,BorderLayout.CENTER);
        statusPanel.setLayout(new BorderLayout());
        statusPanel.add(lblStatus, BorderLayout.CENTER);
        statusPanel.setBorder(new BevelBorder(BevelBorder.LOWERED));
        this.getContentPane().add(statusPanel,BorderLayout.SOUTH);
        setStatus("Ready");
	}
	
	public void displayPanel(int panel){
		setStatus("Loading");
		System.out.println("displayPanl");
		if(currentPanel!=null){
			conPanel.remove(currentPanel);
			//this.getContentPane().removeAll();
		}
		switch(panel){
			case PANEL_MAIN:
				currentPanel=new MainPanel(this);
				break;
			case PANEL_CONFIGURE:
				System.out.println("configure");
				currentPanel=new ConfigPanel(this);
				((ConfigPanel) currentPanel).setConfig(config);
				break;
			case PANEL_ACTION:
				currentPanel=new ActionPanel(this);
				break;
			case PANEL_SSH:
				currentPanel=new SSHPanel(this);
				break;
			case PANEL_MONITOR:
				currentPanel=new MonitorPanel(this);
				break;	
			case PANEL_RESULTS:
				currentPanel=new ResultsPanel(this);
				break;	
		}
		conPanel.add(currentPanel);
		conPanel.revalidate();
		conPanel.repaint();
		setStatus("Ready");
	}
	
	public void setStatus(String str){
			//update frame status
		lblStatus.setText(str);
		repaint();
	}
	
	public void backToMain(){
		displayPanel(PANEL_MAIN);
	}
}

  class MainPanel extends JPanel{
    private Label title1 = new Label("Realtime");
    private Label title2 = new Label("Network Performance");
    private Label title3 = new Label("Forecasting System");

    private Label subtitle1 = new Label("Illinois Institute");
    private Label subtitle2 = new Label("of Technology");
    private static String info = "   This System employs a neural network based approach for predicting the the network bandwidth. The system also supports online traffic monitoring using Network Weather Service. This GUI allows the user to bring up the system for traffic monitoring, forecasting and analyzing the results.";
  
    private JTextArea infoArea= new JTextArea();
    
    private JButton btnStart = new JButton("Start Forcast");
    private JButton btnConfigure = new JButton("Configure System");
    private JButton btnMonitor = new JButton("Monitor traffic");
    private JButton btnChart = new JButton("SSH prompt");
    private GUI gui = null;
    public MainPanel(GUI gui){
    	this.gui = gui;
    	JPanel pnlText= new JPanel();
    	pnlText.setLayout(new GridBagLayout());
	    Insets insets = new Insets(0,0,0,0);
	    Font fntTitle = new Font("Times New Roman",Font.PLAIN,12);
	    title1.setFont(fntTitle);
	    title2.setFont(fntTitle);
	    title3.setFont(fntTitle);
	    pnlText.add(title1,new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.CENTER,GridBagConstraints.NONE,insets,0,0));
	    pnlText.add(title2,new GridBagConstraints(0,1,1,1,0,0,GridBagConstraints.CENTER,GridBagConstraints.NONE,insets,0,0));
	    pnlText.add(title3,new GridBagConstraints(0,2,1,1,0,0,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(0,0,20,0),0,0));
	    pnlText.add(subtitle1,new GridBagConstraints(0,3,1,1,0,0,GridBagConstraints.CENTER,GridBagConstraints.NONE,insets,0,0));
	    pnlText.add(subtitle2,new GridBagConstraints(0,4,1,1,0,0,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(0,0,40,0),0,0));
      
	    infoArea.setText(info);
	    infoArea.setEditable(false); 
	    infoArea.setLineWrap(true);
      	infoArea.setSize(new Dimension(320,250));
      	infoArea.setWrapStyleWord(true);
      	infoArea.setBackground(gui.getBackground());//Color.LIGHT_GRAY);
       	pnlText.add(infoArea,new GridBagConstraints(0,6,1,1,0,0,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(0,0,40,0),0,0));
      	
      	JPanel pnlButton = new JPanel();
      	add(pnlButton,new GridBagConstraints(0,7,1,1,0,0,GridBagConstraints.CENTER,GridBagConstraints.NONE,insets,0,0));
      	pnlButton.setLayout(new GridBagLayout());
      	btnConfigure.setPreferredSize(new Dimension(125,30));
      	btnMonitor.setPreferredSize(new Dimension(125,30));
      	btnStart.setPreferredSize(new Dimension(125,30));
      	btnChart.setPreferredSize(new Dimension(125,30));
      	btnConfigure.addActionListener(btnListener);
      	btnMonitor.addActionListener(btnListener);
      	btnStart.addActionListener(btnListener);
      	btnChart.addActionListener(btnListener);
      	
      	insets = new Insets(10,10,10,10);
      	pnlButton.add(btnConfigure,new GridBagConstraints(0,0,1,1,0,0,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(0,0,0,0),0,0));
      	pnlButton.add(btnMonitor,new GridBagConstraints(0,1,1,1,0,0,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(0,0,0,0),0,0));
      	pnlButton.add(btnStart,new GridBagConstraints(1,0,1,1,0,0,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(0,0,0,0),0,0));
      	pnlButton.add(btnChart,new GridBagConstraints(1,1,1,1,0,0,GridBagConstraints.CENTER,GridBagConstraints.NONE,new Insets(0,0,0,0),0,0));
      	TitledBorder border= BorderFactory.createTitledBorder("Predictor System");
      	pnlButton.setBorder(border);
      	
      	setLayout(new BorderLayout());
      	add(pnlText,BorderLayout.CENTER);
      	add(pnlButton,BorderLayout.SOUTH);
      
    }
    private ActionListener btnListener = new ActionListener(){
	   	public void actionPerformed(ActionEvent e) {
	   		System.out.println(e.getSource());
	   		if(e.getSource() == btnConfigure){
	   			System.out.println(e.getSource());
	   			gui.displayPanel(GUI.PANEL_CONFIGURE);
	   		}
	   		if(e.getSource() == btnStart){
	   			gui.displayPanel(GUI.PANEL_ACTION);
	   		}
	   		if(e.getSource() == btnMonitor){
	   			gui.displayPanel(GUI.PANEL_MONITOR);
	   		}
	   		if(e.getSource() == btnChart){
	   			gui.displayPanel(GUI.PANEL_SSH);
	   		}
	   		
	   	}
   };
  }
  
   
 

