/*

* Copyright (c) 2003-2006 Illinois Institute of Technology.
* All rights reserved.
*
* This file is part of the NBP software package. For license
* information, see the LICENSE file in the top level directory of the
* NBP source distribution.
*
* Released Date: September 5 2006
* This is the NBP I release by the
* NBP Group,
* http://www.cs.iit.edu/~nbp
* Department of Computer Science,
* Illinois Institute of Technology.
*
* This release has been tested under Windows XP Professional
*
* Supervisor
* ----------
* Illiniois Institute of Technology:
* -Dr. Xian-He Sun
*
* Author(s)
* -----------------
* Illinois Institute of Technology:
* -Alaknantha Eswaradass
* -Ming Wu
* -Xian-He Sun
*
*/


/*******************************************************************************************************
 *    @date    : 3/4/2005
 *    @version : 2.0
 *  
 *
 * This code implements a input data processing component
 *
 * Input:   The raw traffic data from NWS  traffic monitoring system
 * Output:  Parsed input data for the ANN
 *
 *
 * The output file created by this program forms the training input for the neural network predictor
*******************************************************************************************************/


import java.util.*;
import java.io.*;


public class ParseInput implements StatusBar{
	
	public void setStatus(String str){
		System.out.println(str);
	}

public ParseInput() {
  }






//This function returns the min, max and the avg values in the given list
  public String[] getMinMaxAvg(List l){
    float value=Float.parseFloat((String)l.get(0));
    float min=value;
    float max=value;
    float avg=0;
    for (int i=0;i<l.size();i++){
      value = Float.parseFloat((String)l.get(i));
      if (value > max)
        max = value;
      if (value < min)
        min = value;
      avg = avg + value;
    }
    avg = avg/l.size();
    String[] results = new String[3];
    results[0]=Float.toString(min);
    results[1]=Float.toString(max);
    results[2]=Float.toString(avg);
    return results;
  }




//This function reads the input file and creates the columns timestamp, min, max and avg no of bits
//in one sec in that time unit and craetes the rows in the difference of binsize specified and
//creates the temp output file
  public void processData(String infile, String annoutfile,  int binsize, StatusBar bar){
    PrintWriter annout = null;
     try{
      String newline = "";
      String line = "";
      String str = "";
      BufferedReader in = new BufferedReader(new FileReader(infile));
      long size =1;
      try{
      	 size = new File(infile).length();
      }catch(Exception ex){
      	ex.printStackTrace();
      }
      annout = new PrintWriter(new FileWriter(annoutfile));
      List rows =new LinkedList();
      line= in.readLine();
      Row row = new Row(line);
      str = row.getValue(0);
      long finished=0;
      int i = 0;
      while(line!=null){
      	finished+=line.length();
      	i++;
      	System.out.println(i+":"+line);
      	if(i==21){
      		System.out.println("break here");
      	}
      	if(i%100 == 0){
	      	int percent = (int) (finished/size * 100);
	      	bar.setStatus("Parsing Phase 1 " + percent + "%");
      	}
        long bin = (Long.parseLong(str.trim()) + binsize);
        rows.clear();
        while ((Long.parseLong(str.trim()) < bin) && (line!=null)) {
          rows.add(row.getValue(4));
          line = in.readLine();
          row = new Row(line);
          if (line != null)
            str = row.getValue(0);
        }

        String[] res = new String[3];
        res = getMinMaxAvg(rows);

        Calendar cal = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
        cal.setTimeInMillis(bin*1000);

        newline = bin + " " + cal.get(Calendar.MONTH) + " " +
                              cal.get(Calendar.DAY_OF_MONTH) + " " +
                              cal.get(Calendar.HOUR_OF_DAY) + " " +
                              cal.get(Calendar.MINUTE) + " " +
                              res[0] + " " + res[1] + " " + res[2];
        annout.println(newline);
       }
      annout.close();
     }
    catch(Exception e){
      e.printStackTrace();
    }finally{
           if(annout!=null) annout.close();
    }
  }





//This function returns the average of totData number of items from the last in the list
public float getAvg(List l, int totData){
	float avg = 0;
	float value=0;
 	for (int i=(l.size()-totData);i<l.size();i++){
       	value = Float.parseFloat((String)l.get(i));
      		avg = avg + value;
    	}
	avg=avg/totData;
	return avg;
}




//This function generates the final input data to be given into the neural network.
//This reads from the temp outfile created  by the function processData ad creates the output
//this puts the columns timestamp, month, day, hour, minute, min no of bits in one sec in that binsize,
//max no of bits in one sec in that binsize, avg no of bits in the past 1 row, 2 rows, ... 6 rows,
//avg no of bits in one sec in that binsize, The value to be predicted in step one
  public void generateFinalInputANN(String infile, String outfile,StatusBar bar,int step){
      PrintWriter out = null;
      String line = "";
      String nextLine = "";
      String newLine="";
      String str = "";
      try{
	      BufferedReader in = new BufferedReader(new FileReader(infile));
	      long size =1;
	      long finished=0;
	      try{
	      	 size = new File(infile).length();
	      }catch(Exception ex){
	      	ex.printStackTrace();
	      }
	      out = new PrintWriter(new FileWriter(outfile));
	      LinkedList rows =new LinkedList();
	      for (int i=0;i<step;i++){
		  	line= in.readLine();
		  	Row row = new Row(line);
			if(line == null) break;
	      		str = row.getValue(7);
			rows.addLast(str);
	      }
	      line=in.readLine();
	      Row row = new Row(line);
	      while(line !=null){
		      nextLine = in.readLine();
		      Row nextRow = new Row(nextLine);
		      if (nextLine == null) break;
	    	  finished+=line.length();
	      	  int percent = (int) (finished/size * 100);
	      	  bar.setStatus("Parsing Phase 1 " + percent + "%");
		      newLine = row.getValue(0)+" "+row.getValue(1)+" "+row.getValue(2)+" "+row.getValue(3)
			  		      + " "+row.getValue(4)+" "+row.getValue(5)+" "+row.getValue(6)
			  	             + " " + Float.toString(getAvg(rows, 1)) + " " + Float.toString(getAvg(rows, 2))
			  	             + " " + Float.toString(getAvg(rows, 3)) + " " + Float.toString(getAvg(rows, 4))
			  	             + " " + Float.toString(getAvg(rows, 5)) + " " + Float.toString(getAvg(rows, 6))
			  	             + " " + row.getValue(7) + " " + nextRow.getValue(7);
			rows.removeFirst();
			rows.addLast(nextRow.getValue(7));
			out.println(newLine);
			line = nextLine;
			row = nextRow;
		}
		out.close();
	}
      catch(Exception e){
         e.printStackTrace();
      }finally{
         if(out!=null) out.close();
      }
  }



public static void main(String argv[]){
	    ParseInput pi=new ParseInput();
	    if(argv.length<3) {
	       System.out.println("Input parameters: input filename, ANN final input filename, binsize");
	       System.exit(-1);
	    }
	    pi.processData(argv[0], "annout.txt",  Integer.parseInt(argv[2]),pi);
	    pi.generateFinalInputANN("annout.txt", argv[1],pi,5);
            //pi.generateNWSResults("nwsout.txt", argv[2]);
   }
}
